from urwid import SimpleFocusListWalker, ListBox, Text
import datetime

class ScrollingListBox(ListBox):

    def __init__(self):
        self.walker = SimpleFocusListWalker([])
        super(ScrollingListBox, self).__init__(self.walker)

    def add(self, message):
        txt = Text(message)
        self.walker.append(txt)
        self.set_focus(len(self.walker)-1)




import scapy.all
from threading import Thread

class Sniffer(object):
    """ Handle sniffing (and effectively reading pcaps) """

    def __init__(self, shark):
        """
        Args:
            shark (scapyshark.ScapyShark): The current scapyshark instance.
        """

        self._shark = shark
        
    def start(self):
        """ Start sniffing. """
        sniffer = Thread(target=scapy.all.sniff, kwargs={'store': False, 'prn': self._handle_new_packet})
        sniffer.daemon = True
        sniffer.start()

    def _handle_new_packet(self, packet):
        self._shark._top_box.add(packet.summary())
        self._shark.loop.draw_screen()



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os, sys, ast

here = os.path.abspath(os.path.dirname(__file__))

#with open(path.join(here, 'README.md'), encoding='utf-8') as f:
#    long_description = f.read()
long_description = "See website for more info."

setup(
    name='scapyshark',
    version='0.0.1',
    description='Wireshark, but with Scapy.',
    long_description=long_description,
    url='https://github.com/bannsec/scapyshark',
    author='Michael Bann',
    author_email='self@bannsecurity.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Operating System :: POSIX :: Linux',
        'Environment :: Console'
    ],
    keywords='wireshark scapy python tcpdump',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['scapy', 'prettytable', 'urwid'],
    entry_points={
        'console_scripts': [
            'scapyshark = scapyshark.scapyshark:main',
        ],
    },
)




#!/usr/bin/env python3

import scapy.all
import urwid
from .scrolling import ScrollingListBox
from .sniffer import Sniffer

palette = [
    ('body','black','dark cyan', 'standout'),
    ('header','light gray', 'black'),
    ('key','light cyan', 'black', 'underline'),
    ('title', 'white', 'black',),
    ('frame', 'dark green', 'black'),
    ]

class ScapyShark(object):

    def __init__(self):
        self._init_window()
        self.sniffer = Sniffer(self)

    def _init_window(self):
        self._header_box = urwid.BoxAdapter(urwid.AttrMap(urwid.Filler(urwid.Text('ScapyShark', align='center')), 'header'), 1)

        # Body
        self._top_box = ScrollingListBox()
        self._middle_box = urwid.ListBox(urwid.SimpleFocusListWalker([urwid.Text('')]))
        self._bottom_box = urwid.ListBox(urwid.SimpleFocusListWalker([urwid.Text('')]))
        
        divider = (1, urwid.Filler(urwid.Divider('\u2500')))
        self._body_pile = urwid.Pile([self._top_box, divider, self._middle_box, divider, self._bottom_box], focus_item=self._top_box)

        # Overarching Frame
        self.frame = urwid.Frame(self._body_pile, header=self._header_box, focus_part='body')

        # The main loop
        self.loop = urwid.MainLoop(urwid.AttrMap(self.frame, 'frame'), unhandled_input=self.unhandled_input, palette=palette)

    def run(self):
        self.sniffer.start()
        self.loop.run()

    def unhandled_input(self, inp):
        if inp in ('q','Q'):
            raise urwid.ExitMainLoop()
        
def main():
    shark = ScapyShark()
    shark.run()


if __name__ == '__main__':
    main()
